@@ -5,10 +5,9 @@ PyTextile
 A Humane Web Text Generator
 """
 
-__version__ = '2.1.0'
-
-__date__ = '2008/11/21'
+__version__ = '2.1.1'
 
+__date__ = '2008/11/30'
 
 __copyright__ = """
 Copyright (c) 2008, Jason Samsa, http://jsamsa.com/
@@ -302,7 +301,9 @@ class Textile(object):
         text = self.retrieve(text)
 
         # Convert to desired output.
-        text = unicode(text, encoding)
+        if isinstance(text, str):
+            text = unicode(text, encoding)
+ 
         text = text.encode(output, 'xmlcharrefreplace')
 
         # Sanitize?
@@ -554,6 +555,7 @@ class Textile(object):
 
         out = []
 
+        anon = False
         for line in text:
             pattern = r'^(%s)(%s%s)\.(\.?)(?::(\S+))? (.*)$' % (tre, self.a, self.c)
             match = re.search(pattern, line, re.S)
@@ -653,7 +655,7 @@ class Textile(object):
             c1 = c2 = ''
 
         elif tag == 'pre':
-            content = self.shelve(self.encode_html(content.rtrim("\n") + "\n"))
+            content = self.shelve(self.encode_html(content.rstrip("\n") + "\n"))
             o1 = "<pre%s>" % atts
             o2 = c2 = ''
             c1 = '</pre>'
@@ -850,51 +852,34 @@ class Textile(object):
         'fooobar ... and hello world ...'
         """
 
-        # (?:^|(?<=[\s>.$pnct\(])|([{[])) # $pre
-        # "                            # start
-        # (' . $this->c . ')           # $atts
-        # ([^"]+)                      # $text
-        # \s?
-        # (?:\(([^)]+)\)(?="))?        # $title
-        # ":
-        # ('.$this->urlch.'+)          # $url
-        # (\/)?                        # $slash
-        # ([^\w\/;]*)                  # $post
-        # (?:([\]}])|(?=\s|$|\)))
+        punct = '!"#$%&\'*+,-./:;=?@\\^_`|~'
 
-
-        # (?:^|(?<=[\s>.$pnct\(])|([{[])) # $pre
-        # "                            # start
-        # ((?:(?:\([^)]+\))|(?:\{[^}]+\})|(?:\[[^]]+\])|(?:\<(?!>)|(?<!<)\>|\<\>|\=|[()]+(?! )))*)           # $atts
-        # ([^"]+)                      # $text
-        # \s?
-        # (?:\(([^)]+)\)(?="))?        # $title
-        # ":
-        # ([\w"$\-_.+!*'(),";\/?:@=&%#{}|\^~\[\]`]+)          # $url
-        # (\/)?                        # $slash
-        # ([^\w\/;]*)                  # $post
-        # (?:([\]}])|(?=\s|$|\)))
-
-        pnct = ".,\"'?!;:"
-        pattern = r"""
-            (?:^|(?<=[\s>.\(])|([{\[])) # pre
-            "                             # start
-            (%s)                          # attrs
-            ([^"]+)                       # text
+        pattern = r'''
+            ([\s\[{(]|[%s])?     # $pre
+            "                          # start
+            (%s)                     # $atts
+            ([^"]+?)                   # $text
             \s?
-            (?:\(([^)]+)\)(?="))?         # title
+            (?:\(([^)]+?)\)(?="))?     # $title
             ":
-            (%s+)                         # url
-            (/)?                          # slash
-            ([^\w\/;]*)                   # post
-            (?:([\]}])|(?=\s|$|\)))
-        """ % (self.c, self.urlch)
-        text = re.compile(pattern, re.U|re.X).sub(self.fLink, text)
+            (\S+?)                     # $url
+            (\/)?                      # $slash
+            ([^\w\/;]*?)               # $post
+            (?=<|\s|$)
+        ''' % (re.escape(punct), self.c)
+
+        text = re.compile(pattern, re.X).sub(self.fLink, text)
 
         return text
 
     def fLink(self, match):
-        pre, atts, text, title, url, slash, post, _ = match.groups()
+        pre, atts, text, title, url, slash, post = match.groups()
+        # print "## ", zip("pre, atts, text, title, url, slash, post".split(","), match.groups()) 
+        # print
+
+        if pre == None:
+            pre = ''
+            
         url = self.checkRefs(url)
 
         atts = self.pba(atts)
@@ -908,10 +893,9 @@ class Textile(object):
 
         url = self.relURL(url)
         if slash: url = url + slash
-        out = '<a href="%s"%s%s>%s</a>%s' % (self.encode_html(url), atts, self.rel, text, post)
-
-        # self.dump(out)
-        return self.shelve(out)
+        out = '<a href="%s"%s%s>%s</a>' % (self.encode_html(url), atts, self.rel, text)
+        out = self.shelve(out)
+        return ''.join([pre, out, post])
 
     def span(self, text):
         """
